package com.jiuqi.util;

import java.text.DecimalFormat;
import java.io.Writer;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

public class Format
{
    public static char fillChar;
    private static final String EOL;
    private static PrintWriter handle;
    private static char alignment;
    private static boolean separator;
    private static char base;
    private static boolean scientific;
    private static boolean zeroFill;
    private static int width;
    private static int decimals;
    private static String pattern;
    
    static {
        Format.fillChar = ' ';
        EOL = System.getProperty("line.separator");
        Format.handle = new PrintWriter(new OutputStreamWriter(System.out));
    }
    
    private Format() {
    }
    
    public static String format(final byte value, final String fd) {
        return formatInteger(value, fd, 8);
    }
    
    public static String format(final char value, final String fd) {
        return formatInteger(value, fd, 16);
    }
    
    public static String format(final double value, final String fd) {
        extractAttributes(fd);
        String s1;
        if (Format.base == 'B') {
            s1 = Long.toBinaryString(Double.doubleToLongBits(value));
            s1 = String.valueOf(repeat(64, '0')) + s1;
            s1 = s1.substring(s1.length() - 64);
        }
        else if (Format.base == 'X') {
            s1 = Long.toHexString(Double.doubleToLongBits(value));
            s1 = String.valueOf(repeat(16, '0')) + s1;
            s1 = s1.substring(s1.length() - 16);
        }
        else {
            Format.pattern = ((Format.decimals != -1) ? ("." + repeat(Format.decimals, '0')) : ".#");
            if (Format.scientific) {
                Format.pattern = "0" + Format.pattern + "E0";
            }
            else {
                Format.pattern = (Format.separator ? ("#,##0" + Format.pattern) : ("0" + Format.pattern));
            }
            s1 = new DecimalFormat(Format.pattern).format(value);
        }
        return size(s1);
    }
    
    public static String format(final float value, final String fd) {
        extractAttributes(fd);
        String s1;
        if (Format.base == 'B') {
            s1 = Integer.toBinaryString(Float.floatToIntBits(value));
            s1 = String.valueOf(repeat(32, '0')) + s1;
            s1 = s1.substring(s1.length() - 32);
        }
        else if (Format.base == 'X') {
            s1 = Integer.toHexString(Float.floatToIntBits(value));
            s1 = String.valueOf(repeat(8, '0')) + s1;
            s1 = s1.substring(s1.length() - 8);
        }
        else {
            Format.pattern = ((Format.decimals != -1) ? ("." + repeat(Format.decimals, '0')) : ".#");
            if (Format.scientific) {
                Format.pattern = "0" + Format.pattern + "E0";
            }
            else {
                Format.pattern = (Format.separator ? ("#,##0" + Format.pattern) : ("0" + Format.pattern));
            }
            s1 = new DecimalFormat(Format.pattern).format(value);
        }
        return size(s1);
    }
    
    public static String format(final int value, final String fd) {
        return formatInteger(value, fd, 32);
    }
    
    public static String format(final long value, final String fd) {
        return formatInteger(value, fd, 64);
    }
    
    public static String format(final short value, final String fd) {
        return formatInteger(value, fd, 16);
    }
    
    private static String formatInteger(final long l, final String s, final int i) {
        extractAttributes(s);
        String s2;
        if (Format.base == 'B') {
            s2 = Long.toBinaryString(l);
            s2 = String.valueOf(repeat(64, '0')) + s2;
            s2 = s2.substring(s2.length() - i);
        }
        else if (Format.base == 'X') {
            s2 = Long.toHexString(l);
            s2 = String.valueOf(repeat(16, '0')) + s2;
            s2 = s2.substring(s2.length() - i / 4);
        }
        else if (Format.separator) {
            s2 = new DecimalFormat("#,###").format(l);
        }
        else {
            s2 = String.valueOf(l);
            if (Format.zeroFill) {
                s2 = String.valueOf(repeat(Format.width - s2.length(), '0')) + s2;
            }
        }
        return size(s2);
    }
    
    private static void extractAttributes(String s) {
        s = s.toUpperCase();
        Format.alignment = 'R';
        Format.separator = false;
        Format.base = 'D';
        Format.scientific = false;
        Format.zeroFill = false;
        Format.width = -1;
        Format.decimals = -1;
        int i = s.indexOf(76, 0);
        if (i > -1) {
            Format.alignment = 'L';
            s = String.valueOf(s.substring(0, i)) + s.substring(i + 1);
        }
        i = s.indexOf(67, 0);
        if (i > -1) {
            Format.alignment = 'C';
            s = String.valueOf(s.substring(0, i)) + s.substring(i + 1);
        }
        i = s.indexOf(44, 0);
        if (i > -1) {
            Format.separator = true;
            Format.pattern = String.valueOf(Format.pattern) + ",###";
            s = String.valueOf(s.substring(0, i)) + s.substring(i + 1);
        }
        i = s.indexOf(88, 0);
        if (i > -1) {
            Format.base = 'X';
            s = String.valueOf(s.substring(0, i)) + s.substring(i + 1);
        }
        else {
            i = s.indexOf(66, 0);
            if (i > -1) {
                Format.base = 'B';
                s = String.valueOf(s.substring(0, i)) + s.substring(i + 1);
            }
        }
        i = s.indexOf(83, 0);
        if (i > -1) {
            Format.scientific = true;
            s = String.valueOf(s.substring(0, i)) + s.substring(i + 1);
        }
        i = s.indexOf(90, 0);
        if (i > -1) {
            Format.zeroFill = true;
            s = String.valueOf(s.substring(0, i)) + s.substring(i + 1);
        }
        i = s.indexOf(46, 0);
        if (i > -1) {
            Format.decimals = Integer.parseInt(s.substring(i + 1));
            s = s.substring(0, i);
        }
        if (s.length() > 0) {
            Format.width = Integer.parseInt(s);
        }
    }
    
    public static void print(final byte value, final String fd) {
        Format.handle.print(format(value, fd));
        Format.handle.flush();
    }
    
    public static void print(final char value, final String fd) {
        Format.handle.print(format(value, fd));
        Format.handle.flush();
    }
    
    public static void print(final double value, final String fd) {
        Format.handle.print(format(value, fd));
        Format.handle.flush();
    }
    
    public static void print(final float value, final String fd) {
        Format.handle.print(format(value, fd));
        Format.handle.flush();
    }
    
    public static void print(final int value, final String fd) {
        Format.handle.print(format(value, fd));
        Format.handle.flush();
    }
    
    public static void print(final long value, final String fd) {
        Format.handle.print(format(value, fd));
        Format.handle.flush();
    }
    
    public static void print(final short value, final String fd) {
        Format.handle.print(format(value, fd));
        Format.handle.flush();
    }
    
    public static void println(final byte value, final String fd) {
        print(value, fd);
        Format.handle.print(Format.EOL);
        Format.handle.flush();
    }
    
    public static void println(final char value, final String fd) {
        print(value, fd);
        Format.handle.print(Format.EOL);
        Format.handle.flush();
    }
    
    public static void println(final double value, final String fd) {
        print(value, fd);
        Format.handle.print(Format.EOL);
        Format.handle.flush();
    }
    
    public static void println(final float value, final String fd) {
        print(value, fd);
        Format.handle.print(Format.EOL);
        Format.handle.flush();
    }
    
    public static void println(final int value, final String fd) {
        print(value, fd);
        Format.handle.print(Format.EOL);
        Format.handle.flush();
    }
    
    public static void println(final long value, final String fd) {
        print(value, fd);
        Format.handle.print(Format.EOL);
        Format.handle.flush();
    }
    
    public static void println(final short value, final String fd) {
        print(value, fd);
        Format.handle.print(Format.EOL);
        Format.handle.flush();
    }
    
    private static String size(final String s) {
        final int i = Format.width - s.length();
        if (Format.alignment == 'R') {
            return String.valueOf(repeat(i, Format.fillChar)) + s;
        }
        if (Format.alignment == 'L') {
            return String.valueOf(s) + repeat(i, Format.fillChar);
        }
        return String.valueOf(repeat(i / 2, Format.fillChar)) + s + repeat(i / 2 + i % 2, Format.fillChar);
    }
    
    public static String repeat(final int times, final char c) {
        String s = "";
        for (int i = 0; i < times; ++i) {
            s = String.valueOf(s) + c;
        }
        return s;
    }
}
